//Written for www.HomoFaciens.de
//Program demonstrates the functionallity of asynchronous motors.
//Copyright (C) 2010 Norbert Heinz
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free //Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or //FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.*;
import javax.swing.*;
import java.applet.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class asynchronous_motor extends JApplet implements  ActionListener, ChangeListener  {

    static String versionNumber="v1.0";

    ScenePainter painter;
    Box dummyBoxCol1 = Box.createHorizontalBox(); //Empty boxes for creating fixed row width and column height
    Box dummyBoxCol3 = Box.createHorizontalBox();
    Box dummyBoxRow1 = Box.createHorizontalBox();
    Box dummyBoxRow2 = Box.createHorizontalBox();
    Box dummyBoxRow3 = Box.createHorizontalBox();
    JSpinner torqueSpinner, sizeSpinner, timeSpinner;
    SpinnerNumberModel torqueModel, sizeModel, timeModel;
    JButton calcButton;
    JLabel torqueLabel, sizeLabel, speedLabel;
    JLabel torqueUnitLabel, sizeUnitLabel, timeUnitLabel;
    JLabel sourceLabel, versionLabel;
    boolean isRunning=false; //is simulation still running?
    Timer timer;             //timer for simulation
    int timerPause=25;       //delay between two calculated frames in milliseconds
    long simulationTime, startTime, simulationTimeOld;  //times passed while simulation is running
    static String language="initalValue";
    String givenParam="nothing yet";   //Arguments passed to applet
    boolean firstPaint=false;          //clears and repaints bufferimage for graphs if true
    String timeName, sizeName, torqueName, startName, stopName;
    Font font = new Font("Arial", Font.PLAIN, 12);

    public void init() {
        UIManager.put("swing.boldMetal", Boolean.FALSE);

        try{
          givenParam = getParameter("LanguageSettings");
        }
        catch(NullPointerException npe){
          System.out.println("could not get parameter 'LanguageSettings'");
        }

        if(givenParam.compareTo("nothing yet")!=0) {
          if (givenParam.compareTo("German")==0)language="German";
          else language="English";
        }
    }

    public void start() {
      int i;
        initComponents();
        timer = new Timer(timerPause, new timerEvent());
    }

    public static void main(String[] args) {
       String programName;

        if(args.length>0){
          if (args[0].compareTo("German")==0){
            language="German";
            programName="Asynchronmotor";
          }
          else{
            language="English";
            programName="Asynchronous motor";
        }
        }
        else{
          language="English";
          programName="Asynchronous motor";
        }

        JFrame f = new JFrame(programName);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JApplet ap = new asynchronous_motor();
        ap.init();
        ap.start();
        f.getContentPane().add("Center", ap);
        f.pack();
        f.setVisible(true);

    }

    public void initComponents() {
        int i;
        GridBagConstraints c = new GridBagConstraints();

        getContentPane().setLayout(new BorderLayout());

        //create panels
        JPanel controlPanel = new JPanel();
        JPanel scenePanel = new JPanel();
        JPanel mainPane = new JPanel();
        JPanel graphPanel = new JPanel();
        JPanel checkBoxPanel = new JPanel();

        //set panel properties
        controlPanel.setLayout(new GridBagLayout());
        mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
        mainPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));


        if(language.compareTo("German")==0){
          torqueName="Drehmoment";
          sizeName="Größe";
          timeName="Zeitfaktor";
          startName="Start";
          stopName="Stop";
        }
        else if(language.compareTo("XXX")==0){
          torqueName="Torque";
        }
        else{
          torqueName="Torque";
          sizeName="Size";
          timeName="Time factor";
          startName="Start";
          stopName="Stop";
        }

        //create buttons, labels etcetera

        calcButton = new JButton(startName);

        torqueLabel = new JLabel(torqueName + ":");
        sizeLabel = new JLabel(sizeName + ":");
        speedLabel = new JLabel(timeName + ":");

        torqueUnitLabel = new JLabel("%");
        sizeUnitLabel = new JLabel("X");
        timeUnitLabel = new JLabel("X");

        sourceLabel = new JLabel("Source & info:    www.HomoFaciens.de");
        versionLabel = new JLabel(versionNumber);

        torqueModel = new SpinnerNumberModel(50.0 , 1.0, 100.0, 1.0);
        timeModel = new SpinnerNumberModel(0.2, 0.01, 1.0, 0.01);
        sizeModel = new SpinnerNumberModel(1, 1, 3, 0.2);


        torqueSpinner = new JSpinner(torqueModel);
        sizeSpinner = new JSpinner(sizeModel);
        timeSpinner = new JSpinner(timeModel);

        sizeSpinner.addChangeListener(this);

        painter = new ScenePainter();

        //set properties of controls

        torqueUnitLabel.setFont(font);
        sizeUnitLabel.setFont(font);

        torqueLabel.setFont(font);
        sizeLabel.setFont(font);

        sourceLabel.setFont(font);
        versionLabel.setFont(font);


        calcButton.setFont(font);
        calcButton.setActionCommand("calcButtonPressed");
        calcButton.addActionListener(this);


        //add components
        c.fill = GridBagConstraints.HORIZONTAL;
        c.ipadx = 10;
        c.insets = new Insets(5,5,5,5);  //padding

        dummyBoxCol1.add(dummyBoxCol1.createRigidArea(new Dimension (130,1)));
        dummyBoxCol3.add(dummyBoxCol3.createRigidArea(new Dimension (30,1)));
        dummyBoxRow1.add(dummyBoxRow1.createRigidArea(new Dimension (1,20)));
        dummyBoxRow2.add(dummyBoxRow2.createRigidArea(new Dimension (1,20)));
        dummyBoxRow3.add(dummyBoxRow3.createRigidArea(new Dimension (1,20)));

        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor=GridBagConstraints.CENTER;

        //set dummyBoxes for fixed layout of columns and rows
        c.gridwidth=1;
        c.gridx = 3;
        c.gridy = 1;
        controlPanel.add(dummyBoxRow1, c);
        c.gridx = 3;
        c.gridy = 2;
        controlPanel.add(dummyBoxRow2, c);
        c.gridx = 0;
        c.gridy = 0;
        controlPanel.add(dummyBoxCol1, c);
        c.gridx = 2;
        c.gridy = 0;
        controlPanel.add(dummyBoxCol3, c);


        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(torqueLabel, c);

        c.gridx = 1;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(torqueSpinner, c);

        c.gridx = 2;
        c.gridy = 1;
        c.gridwidth=1;
        controlPanel.add(torqueUnitLabel, c);

        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(sizeLabel, c);

        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(sizeSpinner, c);

        c.gridx = 2;
        c.gridy = 2;
        c.gridwidth=1;
        controlPanel.add(sizeUnitLabel, c);

        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(speedLabel, c);

        c.gridx = 1;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(timeSpinner, c);

        c.gridx = 2;
        c.gridy = 3;
        c.gridwidth=1;
        controlPanel.add(timeUnitLabel, c);

        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth=3;
        controlPanel.add(calcButton, c);


        c.gridx = 0;
        c.gridy = 5;
        c.gridwidth=3;
        controlPanel.add(checkBoxPanel, c);

        c.gridx = 0;
        c.gridy = 6;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        controlPanel.add(sourceLabel, c);

        c.gridx = 0;
        c.gridy = 7;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        controlPanel.add(versionLabel, c);

        //setup panels
        scenePanel.add("Center", painter);
        graphPanel.add("Center", controlPanel);

        mainPane.add(scenePanel);
        mainPane.add(graphPanel);
        getContentPane().add(mainPane);
        getRootPane().setDefaultButton(calcButton);

    }

    //Listens if values have been changed
    public void stateChanged(ChangeEvent e) {
      painter.scale=sizeModel.getNumber().doubleValue();
      painter.refreshScene();
    }

    // Listens to the buttons.
    public void actionPerformed(ActionEvent e) {

        if("calcButtonPressed".equals(e.getActionCommand())) {

          if(isRunning){
            calcButton.setText(startName);
            isRunning=false;
          }
          else{
            calcButton.setText(stopName);
            startTime=System.currentTimeMillis();
            simulationTime=0;
            simulationTimeOld=0;
            if(timer.getInitialDelay()!=timerPause){
              timer.setInitialDelay(timerPause);
            }
            timer.start();
            isRunning=true;

          }//if(!isRunning)
        }//if ("calcButtonPressed".equals(e.getActionCommand())) {
    }

    private class timerEvent implements ActionListener{
      public void actionPerformed(ActionEvent e){
        simulationTimeOld=simulationTime;
        simulationTime=System.currentTimeMillis()-startTime;

        if(!isRunning){//End simulation
          timer.stop();
          isRunning=false;
          if(language.compareTo("German")==0){
            calcButton.setText("Start");
          }
          else if(language.compareTo("XXX")==0){
            calcButton.setText("Start");
          }
          else{
            calcButton.setText("Start");
          }
        }

        if(isRunning){

            painter.scale=sizeModel.getNumber().doubleValue();
            painter.rotorTorque=torqueModel.getNumber().doubleValue();

            painter.rotorCycle+=(simulationTime-simulationTimeOld)  * timeModel.getNumber().doubleValue() * 0.001 * (100.0-painter.rotorTorque)/100.0;
            painter.fieldCycle+=(simulationTime-simulationTimeOld)  * timeModel.getNumber().doubleValue() * 0.001;

            while(painter.rotorCycle>2.0*3.1415927)painter.rotorCycle-=2.0*3.1415927;
            while(painter.fieldCycle>2.0*3.1415927)painter.fieldCycle-=2.0*3.1415927;

        }//if(isRunning)
        painter.refreshScene();

      }
    }
}


class ScenePainter extends Component {
    BufferedImage bufferImage = null;
    Graphics2D bufferImageSurface=null;
    int sceneHeight = 400, sceneWidth = 400;
    double rotorCycle=0.0, fieldCycle=0.0, rotorTorque=1.0;
    double[] xPoints;
    double[] yPoints;
    int[] xPointsInt;
    int[] yPointsInt;
    int[] xPointsPolygon;
    int[] yPointsPolygon;
    int[] xPointsPolygonRot;
    int[] yPointsPolygonRot;
    int[] xRect1, xRect2, yRect1, yRect2;
    double radius=190.0;
    double scale=1.0;

    public Dimension getPreferredSize(){
        return new Dimension(sceneWidth, sceneHeight);
    }

    void refreshScene() {
      repaint();
    }


    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension size = getSize();
        Composite origComposite;
        int i, i2;
        int xMiddle=sceneWidth/2, yMiddle;  //Coordinates of motor center
        double x;
        double alpha;
        double xRot=0.0, yRot=0.0;
        int redValue=0;
        double currentX1=0.0, currentY1=0.0, currentX2=0.0, currentY2=0.0;
        int currentX1Rot=0, currentY1Rot=0, currentX2Rot=0, currentY2Rot=0;
        int fieldX=0, fieldY=0, fieldXRot=0, fieldYRot=0;
        boolean currentShift;

        //create buffer to avoid flickering while scene is calculated and painted
        if(bufferImage==null){
          bufferImage = (BufferedImage)createImage(sceneWidth, sceneHeight);
          bufferImageSurface = bufferImage.createGraphics();
          xPointsPolygon=new int[13];
          yPointsPolygon=new int[13];
          xPointsPolygonRot=new int[13];
          yPointsPolygonRot=new int[13];
          xRect1=new int[4];
          xRect2=new int[4];
          yRect1=new int[4];
          yRect2=new int[4];
        }

        bufferImageSurface.setBackground(new Color(255, 255, 255));
        bufferImageSurface.clearRect(0, 0, sceneWidth, sceneHeight);
        origComposite = g2.getComposite();

        yMiddle=(int)((double)sceneHeight / 2.0 * scale);
        fieldY=(int)(radius * scale * 1.0);
        fieldX=0;

        //paint stator

        //paint stator case
        bufferImageSurface.setColor(new Color(220,220,220));
        bufferImageSurface.fillOval((int)(xMiddle-radius * scale), (int)(yMiddle-radius * scale), (int)(radius * scale * 2.0), (int)(radius * scale * 2.0));
        bufferImageSurface.setColor(new Color(255,255,255));
        bufferImageSurface.fillOval((int)(xMiddle-radius * scale * 0.94), (int)(yMiddle-radius * scale * 0.94), (int)(radius * scale * 2.0 * 0.94), (int)(radius * scale * 2.0 * 0.94));

        //calculate coordinates for one stator pole

        int statorPoleCount=6;

        for(i=0;i<11;i++){
          x=0.2 - (double)i * 0.2 / 5.0;
          //Rounded side
          yPointsPolygon[i]=(int)(radius * scale * 0.75 * Math.sqrt(1.0 - Math.pow(x, 2.0)));
          xPointsPolygon[i]=(int)(x * radius * scale * 0.75);
        }
        yPointsPolygon[11]=(int)(radius * scale * 0.99);//Top point of stator pole
        xPointsPolygon[11]=-(int)(radius * scale * 0.75 * 0.2);
        yPointsPolygon[12]=(int)(radius * scale * 0.99);//Top point of stator pole
        xPointsPolygon[12]=(int)(radius * scale * 0.75 * 0.2);

        //Draw poles of rotating magnetic field (small red and green dot)
        currentX1=xPointsPolygon[10] + radius * scale * 0.36;
        currentX2=-currentX1;
        currentY1=(int)(radius * scale * 0.84);//Top point of stator pole;
        currentY2=currentY1;

        fieldXRot= (int)(fieldX * Math.cos(-fieldCycle) + fieldY * Math.sin(-fieldCycle)) + xMiddle;
        fieldYRot = -(int)(fieldX * Math.sin(-fieldCycle) + fieldY * Math.cos(-fieldCycle)) + yMiddle;
        bufferImageSurface.setColor(new Color(0,255,0));
        bufferImageSurface.fillOval(fieldXRot - (int)(10 * scale), fieldYRot - (int)(10 * scale), (int)(20 * scale), (int)(20 * scale));

        fieldXRot= (int)(fieldX * Math.cos(-fieldCycle) - fieldY * Math.sin(-fieldCycle)) + xMiddle;
        fieldYRot = -(int)(fieldX * Math.sin(-fieldCycle) - fieldY * Math.cos(-fieldCycle)) + yMiddle;
        bufferImageSurface.setColor(new Color(255,0,0));
        bufferImageSurface.fillOval(fieldXRot - (int)(10 * scale), fieldYRot - (int)(10 * scale), (int)(20 * scale), (int)(20 * scale));

        //Rotate coordinates and paint stator poles
        double angleDiff=0.0;
        bufferImageSurface.setColor(new Color(0,0,0));
        for(i2=0;i2<statorPoleCount;i2++){
          alpha = (double) i2 * 2.0 * 3.1415927 / (double)statorPoleCount;
          for(i=0;i<13;i++){
            xRot = xPointsPolygon[i] * Math.cos(alpha) + yPointsPolygon[i] * Math.sin(alpha);
            yRot = -xPointsPolygon[i] * Math.sin(alpha) + yPointsPolygon[i]* Math.cos(alpha);
            xPointsPolygonRot[i]=(int)xRot + xMiddle;
            yPointsPolygonRot[i]=(int)yRot + yMiddle;
          }

          angleDiff=fieldCycle-alpha;
          if (angleDiff<0.0) angleDiff=-angleDiff;
          if (angleDiff>3.1415927) angleDiff=2.0 * 3.1415927 - angleDiff;
          redValue=255-(int)(255.0/(3.1415927) * angleDiff);
          if(redValue<0)redValue=-redValue;
          bufferImageSurface.setColor(new Color(redValue,255-redValue,0));
          bufferImageSurface.fillPolygon(xPointsPolygonRot, yPointsPolygonRot, 13);

          //Draw current direction of stator windings
          currentX1Rot=(int)(currentX1 * Math.cos(alpha) + currentY1 * Math.sin(alpha)) + xMiddle;
          currentY1Rot = (int)(-currentX1 * Math.sin(alpha) + currentY1 * Math.cos(alpha)) + yMiddle;
          currentX2Rot=(int)(currentX2 * Math.cos(alpha) + currentY2 * Math.sin(alpha)) + xMiddle;
          currentY2Rot = (int)(-currentX2 * Math.sin(alpha) + currentY2 * Math.cos(alpha)) + yMiddle;

          currentShift=false;
          if(angleDiff>3.1415927 / 2.0){
            currentShift=true;
            angleDiff=3.1415927 - angleDiff;
            redValue=(int)(255.0/(3.1415927 / 2.0)*angleDiff);
          }
          else{
            redValue=(int)(255.0/(3.1415927 / 2.0)*angleDiff);
          }
          bufferImageSurface.setColor(new Color(200,100,0));
          bufferImageSurface.drawOval(currentX1Rot - (int)(10 * scale), currentY1Rot - (int)(10 * scale), (int)(20 * scale), (int)(20 * scale));
          bufferImageSurface.setColor(new Color(redValue,redValue,redValue));
          if(currentShift){
            bufferImageSurface.fillOval(currentX1Rot - (int)(2 * scale), currentY1Rot - (int)(2 * scale), (int)(4 * scale), (int)(4 * scale));
          }
          else{
            bufferImageSurface.drawLine(currentX1Rot - (int)(10 * scale * 0.7), currentY1Rot - (int)(10 * scale * 0.7), currentX1Rot + (int)(10 * scale * 0.7), currentY1Rot + (int)(10 * scale * 0.7));
            bufferImageSurface.drawLine(currentX1Rot + (int)(10 * scale * 0.7), currentY1Rot - (int)(10 * scale * 0.7), currentX1Rot - (int)(10 * scale * 0.7), currentY1Rot + (int)(10 * scale * 0.7));
          }
          bufferImageSurface.setColor(new Color(200,100,0));
          bufferImageSurface.drawOval(currentX2Rot - (int)(10 * scale), currentY2Rot - (int)(10 * scale), (int)(20 * scale), (int)(20 * scale));
          bufferImageSurface.setColor(new Color(redValue,redValue,redValue));
          if(currentShift){
            bufferImageSurface.drawLine(currentX2Rot - (int)(10 * scale * 0.7), currentY2Rot - (int)(10 * scale * 0.7), currentX2Rot + (int)(10 * scale * 0.7), currentY2Rot + (int)(10 * scale * 0.7));
            bufferImageSurface.drawLine(currentX2Rot + (int)(10 * scale * 0.7), currentY2Rot - (int)(10 * scale * 0.7), currentX2Rot - (int)(10 * scale * 0.7), currentY2Rot + (int)(10 * scale * 0.7));
          }
          else{
            bufferImageSurface.fillOval(currentX2Rot - (int)(2 * scale), currentY2Rot - (int)(2 * scale), (int)(4 * scale), (int)(4 * scale));
          }
        }//for(i2=0;i2<statorPoleCount;i2++)


        //calculate coordinates for one rotor pole
        int rotorPoleCount=6;

        int xRect01=(int)(radius * scale * 0.6 * 0.2);
        int xRect02=-(int)(radius * scale * 0.6 * 0.2);
        int xRect03=-(int)(radius * scale * 0.6 * 0.2);
        int xRect04=(int)(radius * scale * 0.6 * 0.2);

        int yRect01=0;
        int yRect02=0;
        int yRect03=(int)(radius * scale * 0.6 * 0.6);
        int yRect04=(int)(radius * scale * 0.6 * 0.6);

        for(i=0;i<10;i++){
          x=0.2 - (double)i * 0.044444444;
          //Rounded side
          yPointsPolygon[i]=(int)(radius * scale * 0.7 * Math.sqrt(1.0 - Math.pow(x, 2.0)));
          xPointsPolygon[i]=(int)(x * radius * scale * 0.7);
        }
        yPointsPolygon[10]=(int)(radius * scale * 0.6);//Center point of rotor pole
        xPointsPolygon[10]=xPointsPolygon[9];
        yPointsPolygon[11]=yPointsPolygon[10];//Center point of rotor pole
        xPointsPolygon[11]=xPointsPolygon[0];

        currentX1=xPointsPolygon[9] + scale * 66.0;
        currentX2=-currentX1;
        currentY1=yPointsPolygon[10];
        currentY2=currentY1;


        //Rotate coordinates and paint rotor poles
        for(i2=0;i2<rotorPoleCount;i2++){
          alpha = rotorCycle + (double) i2 * 2.0 * 3.1415927 / (double)rotorPoleCount;
          for(i=0;i<12;i++){
            xRot = xPointsPolygon[i] * Math.cos(alpha) + yPointsPolygon[i] * Math.sin(alpha);
            yRot = -xPointsPolygon[i] * Math.sin(alpha) + yPointsPolygon[i]* Math.cos(alpha);
            xPointsPolygonRot[i]=(int)xRot + xMiddle;
            yPointsPolygonRot[i]=(int)yRot + yMiddle;
          }
          //outer pole of the rotor electromagnet
          xRect1[0] = (int)(xPointsPolygon[11] * Math.cos(alpha) + yPointsPolygon[10] * Math.sin(alpha)) + xMiddle;
          xRect1[1] = (int)(xPointsPolygon[10] * Math.cos(alpha) + yPointsPolygon[10] * Math.sin(alpha)) + xMiddle;
          xRect1[2] = (int)(xPointsPolygon[10] * Math.cos(alpha) + yPointsPolygon[10] * 0.9 * Math.sin(alpha)) + xMiddle;
          xRect1[3] = (int)(xPointsPolygon[11] * Math.cos(alpha) + yPointsPolygon[10] * 0.9 * Math.sin(alpha)) + xMiddle;
          yRect1[0] = (int)(-xPointsPolygon[11] * Math.sin(alpha) + yPointsPolygon[10] * Math.cos(alpha)) + yMiddle;
          yRect1[1] = (int)(-xPointsPolygon[10] * Math.sin(alpha) + yPointsPolygon[10] * Math.cos(alpha)) + yMiddle;
          yRect1[2] = (int)(-xPointsPolygon[10] * Math.sin(alpha) + yPointsPolygon[10] * 0.9 * Math.cos(alpha)) + yMiddle;
          yRect1[3] = (int)(-xPointsPolygon[11] * Math.sin(alpha) + yPointsPolygon[10] * 0.9 * Math.cos(alpha)) + yMiddle;

          //inner pole of the rotor electromagnet
          xRect2[0] = (int)(xPointsPolygon[11] * Math.cos(alpha) + yPointsPolygon[10] * 0.9 * Math.sin(alpha)) + xMiddle;
          xRect2[1] = (int)(xPointsPolygon[10] * Math.cos(alpha) + yPointsPolygon[10] * 0.9 * Math.sin(alpha)) + xMiddle;
          xRect2[2] = (int)(xPointsPolygon[10] * Math.cos(alpha) + 0 * Math.sin(alpha)) + xMiddle;
          xRect2[3] = (int)(xPointsPolygon[11] * Math.cos(alpha) + 0 * Math.sin(alpha)) + xMiddle;
          yRect2[0] = (int)(-xPointsPolygon[11] * Math.sin(alpha) + yPointsPolygon[10] * 0.9 * Math.cos(alpha)) + yMiddle;
          yRect2[1] = (int)(-xPointsPolygon[10] * Math.sin(alpha) + yPointsPolygon[10] * 0.9 * Math.cos(alpha)) + yMiddle;
          yRect2[2] = (int)(-xPointsPolygon[10] * Math.sin(alpha) + 0 * Math.cos(alpha)) + yMiddle;
          yRect2[3] = (int)(-xPointsPolygon[11] * Math.sin(alpha) + 0 * Math.cos(alpha)) + yMiddle;


          currentX1Rot=(int)(currentX1 * Math.cos(alpha) + currentY1 * Math.sin(alpha)) + xMiddle;
          currentY1Rot = (int)(-currentX1 * Math.sin(alpha) + currentY1 * Math.cos(alpha)) + yMiddle;
          currentX2Rot=(int)(currentX2 * Math.cos(alpha) + currentY2 * Math.sin(alpha)) + xMiddle;
          currentY2Rot = (int)(-currentX2 * Math.sin(alpha) + currentY2 * Math.cos(alpha)) + yMiddle;

          angleDiff=alpha - fieldCycle;
          while (angleDiff > 2.0 * 3.1415927) angleDiff-=2.0 * 3.1415927;
          while (angleDiff < 0.0) angleDiff+=2.0 * 3.1415927;
          currentShift=true;
          if (angleDiff>3.1415927) currentShift=false;

          bufferImageSurface.setColor(new Color(200,100,0));
          bufferImageSurface.drawOval(currentX1Rot - (int)(10 * scale), currentY1Rot - (int)(10 * scale), (int)(20 * scale), (int)(20 * scale));
          if(currentShift){
            bufferImageSurface.fillOval(currentX1Rot - (int)(2 * scale), currentY1Rot - (int)(2 * scale), (int)(4 * scale), (int)(4 * scale));
          }
          else{
            bufferImageSurface.drawLine(currentX1Rot - (int)(10 * scale * 0.7), currentY1Rot - (int)(10 * scale * 0.7), currentX1Rot + (int)(10 * scale * 0.7), currentY1Rot + (int)(10 * scale * 0.7));
            bufferImageSurface.drawLine(currentX1Rot + (int)(10 * scale * 0.7), currentY1Rot - (int)(10 * scale * 0.7), currentX1Rot - (int)(10 * scale * 0.7), currentY1Rot + (int)(10 * scale * 0.7));
          }

          bufferImageSurface.drawOval(currentX2Rot - (int)(10 * scale), currentY2Rot - (int)(10 * scale), (int)(20 * scale), (int)(20 * scale));
          if(currentShift){
            bufferImageSurface.drawLine(currentX2Rot - (int)(10 * scale * 0.7), currentY2Rot - (int)(10 * scale * 0.7), currentX2Rot + (int)(10 * scale * 0.7), currentY2Rot + (int)(10 * scale * 0.7));
            bufferImageSurface.drawLine(currentX2Rot + (int)(10 * scale * 0.7), currentY2Rot - (int)(10 * scale * 0.7), currentX2Rot - (int)(10 * scale * 0.7), currentY2Rot + (int)(10 * scale * 0.7));
          }
          else{
            bufferImageSurface.fillOval(currentX2Rot - (int)(2 * scale), currentY2Rot - (int)(2 * scale), (int)(4 * scale), (int)(4 * scale));
          }


          if(angleDiff<3.1415927 / 2.0 || angleDiff > 3.1415027 * 3.0 / 2.0){
            if(angleDiff > 3.1415027 * 3.0 / 2.0){
              angleDiff-=3.1415927 * 3.0 / 2.0;
              redValue=(int)(255.0 / (3.1414597 / 2.0) * angleDiff);
            }
            else{
              redValue=255-(int)(255.0 / (3.1414597 / 2.0) * angleDiff);
            }
          }
          else{
            angleDiff-=3.1415927 / 2.0;
            if(angleDiff > 3.1415027 / 2.0){
              angleDiff-=3.1415927 / 2.0;
              redValue=255-(int)(255.0 / (3.1414597 / 2.0) * angleDiff);
            }
            else{
              redValue=(int)(255.0 / (3.1414597 / 2.0) * angleDiff);
            }
          }
          redValue=(int)((double)(redValue) / 200.0 * rotorTorque + (double)(redValue)/2.0);
          if(currentShift){
            bufferImageSurface.setColor(new Color(redValue,0,0));
          }
          else{
            bufferImageSurface.setColor(new Color(0,redValue,0));
          }
          bufferImageSurface.fillPolygon(xPointsPolygonRot, yPointsPolygonRot, 12);
          if(currentShift){
            bufferImageSurface.setColor(new Color(0,redValue,0));
          }
          else{
            bufferImageSurface.setColor(new Color(redValue,0,0));
          }
          bufferImageSurface.fillPolygon(xRect1, yRect1, 4);
          bufferImageSurface.setColor(new Color(200,200,200));
          bufferImageSurface.fillPolygon(xRect2, yRect2, 4);

        }//for(i2=0;i2<statorPoleCount;i2++)

        //Draw center of rotor
        bufferImageSurface.setColor(new Color(200,200,200));
        bufferImageSurface.fillOval((int)(xMiddle-radius * scale*0.33), (int)(yMiddle-radius * scale*0.33), (int)(radius * scale*0.66), (int)(radius * scale*0.66));
        //Draw rotor axis
        bufferImageSurface.setColor(new Color(0,200,200));
        bufferImageSurface.fillOval((int)(xMiddle-radius * scale*0.02), (int)(yMiddle-radius * scale*0.02), (int)(radius * scale*0.04), (int)(radius * scale*0.04));

        bufferImageSurface.setColor(new Color(0,0,0));
        bufferImageSurface.drawRect(0, 0, sceneWidth-1, sceneHeight-1);

        //paint bufferImage and make it visible
        g2.drawImage(bufferImage, 0, 0, this);

        g2.setComposite(origComposite);

    }
}

